@@ -1,6 +1,6 @@
 /* armv8-sha256.c
  *
- * Copyright (C) 2006-2016 wolfSSL Inc.
+ * Copyright (C) 2006-2020 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -26,7 +26,13 @@
 
 #include <wolfssl/wolfcrypt/settings.h>
 
-#if !defined(NO_SHA256) && defined(WOLFSSL_ARMASM)
+#ifdef WOLFSSL_ARMASM
+#if !defined(NO_SHA256) || defined(WOLFSSL_SHA224)
+
+#ifdef HAVE_FIPS
+#undef HAVE_FIPS
+#endif
+
 #include <wolfssl/wolfcrypt/sha256.h>
 #include <wolfssl/wolfcrypt/logging.h>
 #include <wolfssl/wolfcrypt/error-crypt.h>
@@ -55,7 +61,8 @@ static const ALIGN32 word32 K[64] = {
     0x90BEFFFAL, 0xA4506CEBL, 0xBEF9A3F7L, 0xC67178F2L
 };
 
-int wc_InitSha256(Sha256* sha256)
+
+static int InitSha256(wc_Sha256* sha256)
 {
     int ret = 0;
 
@@ -79,43 +86,39 @@ int wc_InitSha256(Sha256* sha256)
     return ret;
 }
 
-
-static INLINE void AddLength(Sha256* sha256, word32 len)
+static WC_INLINE void AddLength(wc_Sha256* sha256, word32 len)
 {
     word32 tmp = sha256->loLen;
-    if ( (sha256->loLen += len) < tmp)
+    if ((sha256->loLen += len) < tmp)
         sha256->hiLen++;                       /* carry low to high */
 }
 
 
 #ifdef __aarch64__
-/* ARMv8 hardware accleration */
-int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
+
+/* ARMv8 hardware acceleration */
+static WC_INLINE int Sha256Update(wc_Sha256* sha256, const byte* data, word32 len)
 {
     word32 add;
     word32 numBlocks;
 
-    if (sha256 == NULL || (data == NULL && len != 0)) {
-        return BAD_FUNC_ARG;
-    }
-
     /* only perform actions if a buffer is passed in */
     if (len > 0) {
         /* fill leftover buffer with data */
-        add = min(len, SHA256_BLOCK_SIZE - sha256->buffLen);
+        add = min(len, WC_SHA256_BLOCK_SIZE - sha256->buffLen);
         XMEMCPY((byte*)(sha256->buffer) + sha256->buffLen, data, add);
         sha256->buffLen += add;
         data            += add;
         len             -= add;
 
         /* number of blocks in a row to complete */
-        numBlocks = (len + sha256->buffLen)/SHA256_BLOCK_SIZE;
+        numBlocks = (len + sha256->buffLen)/WC_SHA256_BLOCK_SIZE;
 
         if (numBlocks > 0) {
             word32* k = (word32*)K;
 
             /* get leftover amount after blocks */
-            add = (len + sha256->buffLen) - numBlocks * SHA256_BLOCK_SIZE;
+            add = (len + sha256->buffLen) - numBlocks * WC_SHA256_BLOCK_SIZE;
             __asm__ volatile (
             "#load leftover data\n"
             "LD1 {v0.2d-v3.2d}, %[buffer]   \n"
@@ -136,7 +139,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
             "LD1 {v24.4s-v27.4s}, [%[k]], #64    \n"
             "LD1 {v28.4s-v31.4s}, [%[k]], #64    \n"
 
-            /* begining of SHA256 block operation */
+            /* beginning of SHA256 block operation */
             "1:\n"
             /* Round 1 */
             "MOV v4.16b, v0.16b        \n"
@@ -267,7 +270,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
             "#check if more blocks should be done\n"
             "CBZ w8, 2f \n"
 
-            "#load in message and schedual updates \n"
+            "#load in message and schedule updates \n"
             "LD1 {v0.2d-v3.2d}, [%[dataIn]], #64   \n"
             "MOV v14.16b, v12.16b \n"
             "MOV v15.16b, v13.16b \n"
@@ -291,7 +294,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
                               "v29", "v30", "v31", "w8"
             );
 
-            AddLength(sha256, SHA256_BLOCK_SIZE * numBlocks);
+            AddLength(sha256, WC_SHA256_BLOCK_SIZE * numBlocks);
 
             /* copy over any remaining data leftover */
             XMEMCPY(sha256->buffer, data, add);
@@ -299,7 +302,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
         }
     }
 
-    /* account for possiblity of not used if len = 0 */
+    /* account for possibility of not used if len = 0 */
     (void)add;
     (void)numBlocks;
 
@@ -307,24 +310,20 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
 }
 
 
-int wc_Sha256Final(Sha256* sha256, byte* hash)
+static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
 {
     byte* local;
 
-    if (sha256 == NULL || hash == NULL) {
-        return BAD_FUNC_ARG;
-    }
-
     local = (byte*)sha256->buffer;
     AddLength(sha256, sha256->buffLen);  /* before adding pads */
 
     local[sha256->buffLen++] = 0x80;     /* add 1 */
 
     /* pad with zeros */
-    if (sha256->buffLen > SHA256_PAD_SIZE) {
+    if (sha256->buffLen > WC_SHA256_PAD_SIZE) {
 
-        XMEMSET(&local[sha256->buffLen], 0, SHA256_BLOCK_SIZE - sha256->buffLen);
-        sha256->buffLen += SHA256_BLOCK_SIZE - sha256->buffLen;
+        XMEMSET(&local[sha256->buffLen], 0, WC_SHA256_BLOCK_SIZE - sha256->buffLen);
+        sha256->buffLen += WC_SHA256_BLOCK_SIZE - sha256->buffLen;
         __asm__ volatile (
             "LD1 {v4.2d-v7.2d}, %[buffer]          \n"
             "MOV v0.16b, v4.16b                    \n"
@@ -476,7 +475,7 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
 
         sha256->buffLen = 0;
     }
-    XMEMSET(&local[sha256->buffLen], 0, SHA256_PAD_SIZE - sha256->buffLen);
+    XMEMSET(&local[sha256->buffLen], 0, WC_SHA256_PAD_SIZE - sha256->buffLen);
 
     /* put lengths in bits */
     sha256->hiLen = (sha256->loLen >> (8*sizeof(sha256->loLen) - 3)) +
@@ -498,12 +497,12 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
         );
     #endif
     /* ! length ordering dependent on digest endian type ! */
-    XMEMCPY(&local[SHA256_PAD_SIZE], &sha256->hiLen, sizeof(word32));
-    XMEMCPY(&local[SHA256_PAD_SIZE + sizeof(word32)], &sha256->loLen,
+    XMEMCPY(&local[WC_SHA256_PAD_SIZE], &sha256->hiLen, sizeof(word32));
+    XMEMCPY(&local[WC_SHA256_PAD_SIZE + sizeof(word32)], &sha256->loLen,
             sizeof(word32));
 
     __asm__ volatile (
-        "#load in message and schedual updates \n"
+        "#load in message and schedule updates \n"
         "LD1 {v4.2d-v7.2d}, %[buffer]        \n"
         "MOV v0.16b, v4.16b \n"
         "MOV v1.16b, v5.16b \n"
@@ -654,37 +653,34 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
                               "v22", "v23", "v24", "v25"
     );
 
-    return wc_InitSha256(sha256);  /* reset state */
+    return 0;
 }
 
 #else /* not using 64 bit */
-/* ARMv8 hardware accleration Aarch32 */
-int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
+
+/* ARMv8 hardware acceleration Aarch32 */
+static WC_INLINE int Sha256Update(wc_Sha256* sha256, const byte* data, word32 len)
 {
     word32 add;
     word32 numBlocks;
 
-    if (sha256 == NULL || (data == NULL && len != 0)) {
-        return BAD_FUNC_ARG;
-    }
-
     /* only perform actions if a buffer is passed in */
     if (len > 0) {
         /* fill leftover buffer with data */
-        add = min(len, SHA256_BLOCK_SIZE - sha256->buffLen);
+        add = min(len, WC_SHA256_BLOCK_SIZE - sha256->buffLen);
         XMEMCPY((byte*)(sha256->buffer) + sha256->buffLen, data, add);
         sha256->buffLen += add;
         data            += add;
         len             -= add;
 
         /* number of blocks in a row to complete */
-        numBlocks = (len + sha256->buffLen)/SHA256_BLOCK_SIZE;
+        numBlocks = (len + sha256->buffLen)/WC_SHA256_BLOCK_SIZE;
 
         if (numBlocks > 0) {
             word32* bufPt = sha256->buffer;
             word32* digPt = sha256->digest;
             /* get leftover amount after blocks */
-            add = (len + sha256->buffLen) - numBlocks * SHA256_BLOCK_SIZE;
+            add = (len + sha256->buffLen) - numBlocks * WC_SHA256_BLOCK_SIZE;
             __asm__ volatile (
             "#load leftover data\n"
             "VLDM %[buffer]!, {q0-q3} \n"
@@ -702,7 +698,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
             "VMOV.32 q14, q12 \n" /* store digest for add at the end */
             "VMOV.32 q15, q13 \n"
 
-            /* begining of SHA256 block operation */
+            /* beginning of SHA256 block operation */
             "1:\n"
 
             /* Round 1 */
@@ -846,7 +842,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
             "CMP r8, #0 \n"
             "BEQ 2f \n"
 
-            "#load in message and schedual updates \n"
+            "#load in message and schedule updates \n"
             "VLD1.32 {q0}, [%[dataIn]]!   \n"
             "VLD1.32 {q1}, [%[dataIn]]!   \n"
             "VLD1.32 {q2}, [%[dataIn]]!   \n"
@@ -874,7 +870,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
                               "q15", "r8"
             );
 
-            AddLength(sha256, SHA256_BLOCK_SIZE * numBlocks);
+            AddLength(sha256, WC_SHA256_BLOCK_SIZE * numBlocks);
 
             /* copy over any remaining data leftover */
             XMEMCPY(sha256->buffer, data, add);
@@ -882,7 +878,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
         }
     }
 
-    /* account for possiblity of not used if len = 0 */
+    /* account for possibility of not used if len = 0 */
     (void)add;
     (void)numBlocks;
 
@@ -890,7 +886,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
 }
 
 
-int wc_Sha256Final(Sha256* sha256, byte* hash)
+static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
 {
     byte* local;
 
@@ -904,11 +900,11 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
     local[sha256->buffLen++] = 0x80;     /* add 1 */
 
     /* pad with zeros */
-    if (sha256->buffLen > SHA256_PAD_SIZE) {
+    if (sha256->buffLen > WC_SHA256_PAD_SIZE) {
         word32* bufPt = sha256->buffer;
         word32* digPt = sha256->digest;
-        XMEMSET(&local[sha256->buffLen], 0, SHA256_BLOCK_SIZE - sha256->buffLen);
-        sha256->buffLen += SHA256_BLOCK_SIZE - sha256->buffLen;
+        XMEMSET(&local[sha256->buffLen], 0, WC_SHA256_BLOCK_SIZE - sha256->buffLen);
+        sha256->buffLen += WC_SHA256_BLOCK_SIZE - sha256->buffLen;
         __asm__ volatile (
         "#load leftover data\n"
         "VLDM %[buffer]!, {q0-q3} \n"
@@ -924,7 +920,7 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
         "VMOV.32 q14, q12 \n" /* store digest for add at the end */
         "VMOV.32 q15, q13 \n"
 
-        /* begining of SHA256 block operation */
+        /* beginning of SHA256 block operation */
         /* Round 1 */
         "VLD1.32 {q5}, [%[k]]!    \n"
         "VMOV.32 q4, q0           \n"
@@ -1079,7 +1075,7 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
 
         sha256->buffLen = 0;
     }
-    XMEMSET(&local[sha256->buffLen], 0, SHA256_PAD_SIZE - sha256->buffLen);
+    XMEMSET(&local[sha256->buffLen], 0, WC_SHA256_PAD_SIZE - sha256->buffLen);
 
     /* put lengths in bits */
     sha256->hiLen = (sha256->loLen >> (8*sizeof(sha256->loLen) - 3)) +
@@ -1087,8 +1083,9 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
     sha256->loLen = sha256->loLen << 3;
 
     /* store lengths */
-	word32* bufPt = sha256->buffer;
     #if defined(LITTLE_ENDIAN_ORDER)
+    {
+        word32* bufPt = sha256->buffer;
         __asm__ volatile (
             "VLD1.32 {q0}, [%[in]] \n"
             "VREV32.8 q0, q0 \n"
@@ -1106,13 +1103,14 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
             : [in] "0" (bufPt)
             : "cc", "memory", "q0", "q1", "q2", "q3"
         );
+    }
     #endif
     /* ! length ordering dependent on digest endian type ! */
-    XMEMCPY(&local[SHA256_PAD_SIZE], &sha256->hiLen, sizeof(word32));
-    XMEMCPY(&local[SHA256_PAD_SIZE + sizeof(word32)], &sha256->loLen,
+    XMEMCPY(&local[WC_SHA256_PAD_SIZE], &sha256->hiLen, sizeof(word32));
+    XMEMCPY(&local[WC_SHA256_PAD_SIZE + sizeof(word32)], &sha256->loLen,
             sizeof(word32));
 
-    bufPt = sha256->buffer;
+    word32* bufPt = sha256->buffer;
     word32* digPt = sha256->digest;
     __asm__ volatile (
         "#load leftover data\n"
@@ -1124,7 +1122,7 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
         "VMOV.32 q14, q12 \n" /* store digest for add at the end */
         "VMOV.32 q15, q13 \n"
 
-        /* begining of SHA256 block operation */
+        /* beginning of SHA256 block operation */
         /* Round 1 */
         "VLD1.32 {q5}, [%[k]]!    \n"
         "VMOV.32 q4, q0           \n"
@@ -1285,9 +1283,251 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
                           "q15"
     );
 
-    return wc_InitSha256(sha256);  /* reset state */
+    return 0;
 }
+
 #endif /* __aarch64__ */
 
-#endif /* NO_SHA256 and WOLFSSL_ARMASM */
 
+#ifndef NO_SHA256
+
+int wc_InitSha256_ex(wc_Sha256* sha256, void* heap, int devId)
+{
+    if (sha256 == NULL)
+        return BAD_FUNC_ARG;
+
+    sha256->heap = heap;
+    (void)devId;
+
+    return InitSha256(sha256);
+}
+
+int wc_InitSha256(wc_Sha256* sha256)
+{
+    return wc_InitSha256_ex(sha256, NULL, INVALID_DEVID);
+}
+
+void wc_Sha256Free(wc_Sha256* sha256)
+{
+    (void)sha256;
+}
+
+int wc_Sha256Update(wc_Sha256* sha256, const byte* data, word32 len)
+{
+    if (sha256 == NULL || (data == NULL && len != 0)) {
+        return BAD_FUNC_ARG;
+    }
+
+    return Sha256Update(sha256, data, len);
+}
+
+int wc_Sha256FinalRaw(wc_Sha256* sha256, byte* hash)
+{
+#ifdef LITTLE_ENDIAN_ORDER
+    word32 digest[WC_SHA256_DIGEST_SIZE / sizeof(word32)];
+#endif
+
+    if (sha256 == NULL || hash == NULL) {
+        return BAD_FUNC_ARG;
+    }
+
+#ifdef LITTLE_ENDIAN_ORDER
+    ByteReverseWords((word32*)digest, (word32*)sha256->digest,
+                                                        WC_SHA256_DIGEST_SIZE);
+    XMEMCPY(hash, digest, WC_SHA256_DIGEST_SIZE);
+#else
+    XMEMCPY(hash, sha256->digest, WC_SHA256_DIGEST_SIZE);
+#endif
+
+    return 0;
+}
+
+int wc_Sha256Final(wc_Sha256* sha256, byte* hash)
+{
+    int ret;
+
+    if (sha256 == NULL || hash == NULL) {
+        return BAD_FUNC_ARG;
+    }
+
+    ret = Sha256Final(sha256, hash);
+    if (ret != 0)
+        return ret;
+
+    return InitSha256(sha256);  /* reset state */
+}
+
+int wc_Sha256GetHash(wc_Sha256* sha256, byte* hash)
+{
+    int ret;
+    wc_Sha256 tmpSha256;
+
+    if (sha256 == NULL || hash == NULL)
+        return BAD_FUNC_ARG;
+
+    ret = wc_Sha256Copy(sha256, &tmpSha256);
+    if (ret == 0) {
+        ret = wc_Sha256Final(&tmpSha256, hash);
+    }
+    return ret;
+}
+
+#if defined(WOLFSSL_HASH_FLAGS) || defined(WOLF_CRYPTO_CB)
+int wc_Sha256SetFlags(wc_Sha256* sha256, word32 flags)
+{
+    if (sha256) {
+        sha256->flags = flags;
+    }
+    return 0;
+}
+int wc_Sha256GetFlags(wc_Sha256* sha256, word32* flags)
+{
+    if (sha256 && flags) {
+        *flags = sha256->flags;
+    }
+    return 0;
+}
+#endif
+
+int wc_Sha256Copy(wc_Sha256* src, wc_Sha256* dst)
+{
+    int ret = 0;
+
+    if (src == NULL || dst == NULL)
+        return BAD_FUNC_ARG;
+
+    XMEMCPY(dst, src, sizeof(wc_Sha256));
+
+    return ret;
+}
+
+#endif /* !NO_SHA256 */
+
+
+#ifdef WOLFSSL_SHA224
+    static int InitSha224(wc_Sha224* sha224)
+    {
+
+        int ret = 0;
+
+        if (sha224 == NULL) {
+            return BAD_FUNC_ARG;
+        }
+
+        sha224->digest[0] = 0xc1059ed8;
+        sha224->digest[1] = 0x367cd507;
+        sha224->digest[2] = 0x3070dd17;
+        sha224->digest[3] = 0xf70e5939;
+        sha224->digest[4] = 0xffc00b31;
+        sha224->digest[5] = 0x68581511;
+        sha224->digest[6] = 0x64f98fa7;
+        sha224->digest[7] = 0xbefa4fa4;
+
+        sha224->buffLen = 0;
+        sha224->loLen   = 0;
+        sha224->hiLen   = 0;
+
+        return ret;
+    }
+
+    int wc_InitSha224_ex(wc_Sha224* sha224, void* heap, int devId)
+    {
+        if (sha224 == NULL)
+            return BAD_FUNC_ARG;
+
+        sha224->heap = heap;
+        (void)devId;
+
+        return InitSha224(sha224);
+    }
+
+    int wc_InitSha224(wc_Sha224* sha224)
+    {
+        return wc_InitSha224_ex(sha224, NULL, INVALID_DEVID);
+    }
+
+    int wc_Sha224Update(wc_Sha224* sha224, const byte* data, word32 len)
+    {
+        int ret;
+
+        if (sha224 == NULL || (data == NULL && len > 0)) {
+            return BAD_FUNC_ARG;
+        }
+
+        ret = Sha256Update((wc_Sha256 *)sha224, data, len);
+
+        return ret;
+    }
+
+    int wc_Sha224Final(wc_Sha224* sha224, byte* hash)
+    {
+        int ret;
+        word32 hashTmp[WC_SHA256_DIGEST_SIZE/sizeof(word32)];
+
+        if (sha224 == NULL || hash == NULL) {
+            return BAD_FUNC_ARG;
+        }
+
+        ret = Sha256Final((wc_Sha256*)sha224, (byte*)hashTmp);
+        if (ret != 0)
+            return ret;
+
+        XMEMCPY(hash, hashTmp, WC_SHA224_DIGEST_SIZE);
+
+        return InitSha224(sha224);  /* reset state */
+    }
+
+    void wc_Sha224Free(wc_Sha224* sha224)
+    {
+        if (sha224 == NULL)
+            return;
+    }
+
+    int wc_Sha224GetHash(wc_Sha224* sha224, byte* hash)
+    {
+        int ret;
+        wc_Sha224 tmpSha224;
+
+        if (sha224 == NULL || hash == NULL)
+            return BAD_FUNC_ARG;
+
+        ret = wc_Sha224Copy(sha224, &tmpSha224);
+        if (ret == 0) {
+            ret = wc_Sha224Final(&tmpSha224, hash);
+        }
+        return ret;
+    }
+
+#if defined(WOLFSSL_HASH_FLAGS) || defined(WOLF_CRYPTO_CB)
+    int wc_Sha224SetFlags(wc_Sha224* sha224, word32 flags)
+    {
+        if (sha224) {
+            sha224->flags = flags;
+        }
+        return 0;
+    }
+    int wc_Sha224GetFlags(wc_Sha224* sha224, word32* flags)
+    {
+        if (sha224 && flags) {
+            *flags = sha224->flags;
+        }
+        return 0;
+    }
+#endif
+
+    int wc_Sha224Copy(wc_Sha224* src, wc_Sha224* dst)
+    {
+        int ret = 0;
+
+        if (src == NULL || dst == NULL)
+            return BAD_FUNC_ARG;
+
+        XMEMCPY(dst, src, sizeof(wc_Sha224));
+
+        return ret;
+    }
+
+#endif /* WOLFSSL_SHA224 */
+
+#endif /* !NO_SHA256 || WOLFSSL_SHA224 */
+#endif /* WOLFSSL_ARMASM */
