@@ -1,6 +1,6 @@
 /* srp.c
  *
- * Copyright (C) 2006-2016 wolfSSL Inc.
+ * Copyright (C) 2006-2020 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -157,28 +157,28 @@ static word32 SrpHashSize(SrpType type)
     switch (type) {
         case SRP_TYPE_SHA:
             #ifndef NO_SHA
-                return SHA_DIGEST_SIZE;
+                return WC_SHA_DIGEST_SIZE;
             #else
                 return 0;
             #endif
 
         case SRP_TYPE_SHA256:
             #ifndef NO_SHA256
-                return SHA256_DIGEST_SIZE;
+                return WC_SHA256_DIGEST_SIZE;
             #else
                 return 0;
             #endif
 
         case SRP_TYPE_SHA384:
             #ifdef WOLFSSL_SHA384
-                return SHA384_DIGEST_SIZE;
+                return WC_SHA384_DIGEST_SIZE;
             #else
                 return 0;
             #endif
 
         case SRP_TYPE_SHA512:
             #ifdef WOLFSSL_SHA512
-                return SHA512_DIGEST_SIZE;
+                return WC_SHA512_DIGEST_SIZE;
             #else
                 return 0;
             #endif
@@ -188,6 +188,35 @@ static word32 SrpHashSize(SrpType type)
     }
 }
 
+static void SrpHashFree(SrpHash* hash)
+{
+    switch (hash->type) {
+        case SRP_TYPE_SHA:
+        #ifndef NO_SHA
+            wc_ShaFree(&hash->data.sha);
+        #endif
+            break;
+        case SRP_TYPE_SHA256:
+        #ifndef NO_SHA256
+            wc_Sha256Free(&hash->data.sha256);
+        #endif
+            break;
+        case SRP_TYPE_SHA384:
+        #ifdef WOLFSSL_SHA384
+            wc_Sha384Free(&hash->data.sha384);
+        #endif
+            break;
+        case SRP_TYPE_SHA512:
+        #ifdef WOLFSSL_SHA512
+            wc_Sha512Free(&hash->data.sha512);
+        #endif
+            break;
+        default:
+            break;
+    }
+}
+
+
 int wc_SrpInit(Srp* srp, SrpType type, SrpSide side)
 {
     int r;
@@ -234,18 +263,21 @@ int wc_SrpInit(Srp* srp, SrpType type, SrpSide side)
     }
 
     /* initializing variables */
-
     XMEMSET(srp, 0, sizeof(Srp));
 
     if ((r = SrpHashInit(&srp->client_proof, type)) != 0)
         return r;
 
-    if ((r = SrpHashInit(&srp->server_proof, type)) != 0)
+    if ((r = SrpHashInit(&srp->server_proof, type)) != 0) {
+        SrpHashFree(&srp->client_proof);
         return r;
-
+    }
     if ((r = mp_init_multi(&srp->N,    &srp->g, &srp->auth,
-                           &srp->priv, 0, 0)) != 0)
+                           &srp->priv, 0, 0)) != 0) {
+        SrpHashFree(&srp->client_proof);
+        SrpHashFree(&srp->server_proof);
         return r;
+    }
 
     srp->side = side;    srp->type   = type;
     srp->salt = NULL;    srp->saltSz = 0;
@@ -269,7 +301,6 @@ void wc_SrpTerm(Srp* srp)
     if (srp) {
         mp_clear(&srp->N);    mp_clear(&srp->g);
         mp_clear(&srp->auth); mp_clear(&srp->priv);
-
         if (srp->salt) {
             ForceZero(srp->salt, srp->saltSz);
             XFREE(srp->salt, srp->heap, DYNAMIC_TYPE_SRP);
@@ -283,6 +314,8 @@ void wc_SrpTerm(Srp* srp)
             XFREE(srp->key, srp->heap, DYNAMIC_TYPE_SRP);
         }
 
+        SrpHashFree(&srp->client_proof);
+        SrpHashFree(&srp->server_proof);
         ForceZero(srp, sizeof(Srp));
     }
 }
@@ -292,12 +325,14 @@ int wc_SrpSetUsername(Srp* srp, const byte* username, word32 size)
     if (!srp || !username)
         return BAD_FUNC_ARG;
 
-    srp->user = (byte*)XMALLOC(size, srp->heap, DYNAMIC_TYPE_SRP);
+    /* +1 for NULL char */
+    srp->user = (byte*)XMALLOC(size + 1, srp->heap, DYNAMIC_TYPE_SRP);
     if (srp->user == NULL)
         return MEMORY_E;
 
     srp->userSz = size;
     XMEMCPY(srp->user, username, srp->userSz);
+    srp->user[size] = '\0';
 
     return 0;
 }
@@ -349,10 +384,12 @@ int wc_SrpSetParams(Srp* srp, const byte* N,    word32 nSz,
     /* Set k = H(N, g) */
             r = SrpHashInit(&hash, srp->type);
     if (!r) r = SrpHashUpdate(&hash, (byte*) N, nSz);
-    for (i = 0; (word32)i < nSz - gSz; i++)
-        SrpHashUpdate(&hash, &pad, 1);
+    for (i = 0; (word32)i < nSz - gSz; i++) {
+        if (!r) r = SrpHashUpdate(&hash, &pad, 1);
+    }
     if (!r) r = SrpHashUpdate(&hash, (byte*) g, gSz);
     if (!r) r = SrpHashFinal(&hash, srp->k);
+    SrpHashFree(&hash);
 
     /* update client proof */
 
@@ -360,11 +397,13 @@ int wc_SrpSetParams(Srp* srp, const byte* N,    word32 nSz,
     if (!r) r = SrpHashInit(&hash, srp->type);
     if (!r) r = SrpHashUpdate(&hash, (byte*) N, nSz);
     if (!r) r = SrpHashFinal(&hash, digest1);
+    SrpHashFree(&hash);
 
     /* digest2 = H(g) */
     if (!r) r = SrpHashInit(&hash, srp->type);
     if (!r) r = SrpHashUpdate(&hash, (byte*) g, gSz);
     if (!r) r = SrpHashFinal(&hash, digest2);
+    SrpHashFree(&hash);
 
     /* digest1 = H(N) ^ H(g) */
     if (r == 0) {
@@ -376,6 +415,7 @@ int wc_SrpSetParams(Srp* srp, const byte* N,    word32 nSz,
     if (!r) r = SrpHashInit(&hash, srp->type);
     if (!r) r = SrpHashUpdate(&hash, srp->user, srp->userSz);
     if (!r) r = SrpHashFinal(&hash, digest2);
+    SrpHashFree(&hash);
 
     /* client proof = H( H(N) ^ H(g) | H(user) | salt) */
     if (!r) r = SrpHashUpdate(&srp->client_proof, digest1, j);
@@ -406,12 +446,14 @@ int wc_SrpSetPassword(Srp* srp, const byte* password, word32 size)
     if (!r) r = SrpHashUpdate(&hash, (const byte*) ":", 1);
     if (!r) r = SrpHashUpdate(&hash, password, size);
     if (!r) r = SrpHashFinal(&hash, digest);
+    SrpHashFree(&hash);
 
     /* digest = H(salt | H(username | ':' | password)) */
     if (!r) r = SrpHashInit(&hash, srp->type);
     if (!r) r = SrpHashUpdate(&hash, srp->salt, srp->saltSz);
     if (!r) r = SrpHashUpdate(&hash, digest, digestSz);
     if (!r) r = SrpHashFinal(&hash, digest);
+    SrpHashFree(&hash);
 
     /* Set x (private key) */
     if (!r) r = mp_read_unsigned_bin(&srp->auth, digest, digestSz);
@@ -552,6 +594,8 @@ static int wc_SrpSetKey(Srp* srp, byte* secret, word32 size)
     byte counter[4];
     int r = BAD_FUNC_ARG;
 
+    XMEMSET(digest, 0, SRP_MAX_DIGEST_SIZE);
+
     srp->key = (byte*)XMALLOC(2 * digestSz, srp->heap, DYNAMIC_TYPE_SRP);
     if (srp->key == NULL)
         return MEMORY_E;
@@ -568,7 +612,7 @@ static int wc_SrpSetKey(Srp* srp, byte* secret, word32 size)
         if (!r) r = SrpHashUpdate(&hash, secret, size);
         if (!r) r = SrpHashUpdate(&hash, counter, 4);
 
-        if(j + digestSz > srp->keySz) {
+        if (j + digestSz > srp->keySz) {
             if (!r) r = SrpHashFinal(&hash, digest);
             XMEMCPY(srp->key + j, digest, srp->keySz - j);
             j = srp->keySz;
@@ -577,6 +621,7 @@ static int wc_SrpSetKey(Srp* srp, byte* secret, word32 size)
             if (!r) r = SrpHashFinal(&hash, srp->key + j);
             j += digestSz;
         }
+        SrpHashFree(&hash);
     }
 
     ForceZero(digest, sizeof(digest));
@@ -588,113 +633,233 @@ static int wc_SrpSetKey(Srp* srp, byte* secret, word32 size)
 int wc_SrpComputeKey(Srp* srp, byte* clientPubKey, word32 clientPubKeySz,
                                byte* serverPubKey, word32 serverPubKeySz)
 {
-    SrpHash hash;
-    byte *secret;
+#ifdef WOLFSSL_SMALL_STACK
+    SrpHash *hash = (SrpHash *)XMALLOC(sizeof *hash, srp->heap, DYNAMIC_TYPE_SRP);
+    byte *digest = (byte *)XMALLOC(SRP_MAX_DIGEST_SIZE, srp->heap, DYNAMIC_TYPE_SRP);
+    mp_int *u = (mp_int *)XMALLOC(sizeof *u, srp->heap, DYNAMIC_TYPE_SRP);
+    mp_int *s = (mp_int *)XMALLOC(sizeof *s, srp->heap, DYNAMIC_TYPE_SRP);
+    mp_int *temp1 = (mp_int *)XMALLOC(sizeof *temp1, srp->heap, DYNAMIC_TYPE_SRP);
+    mp_int *temp2 = (mp_int *)XMALLOC(sizeof *temp2, srp->heap, DYNAMIC_TYPE_SRP);
+#else
+    SrpHash hash[1];
     byte digest[SRP_MAX_DIGEST_SIZE];
+    mp_int u[1], s[1], temp1[1], temp2[1];
+#endif
+    byte *secret = NULL;
     word32 i, secretSz, digestSz;
-    mp_int u, s, temp1, temp2;
     byte pad = 0;
     int r;
 
     /* validating params */
 
-    if (!srp || !clientPubKey || clientPubKeySz == 0
-             || !serverPubKey || serverPubKeySz == 0)
-        return BAD_FUNC_ARG;
+    if ((mp_init_multi(u, s, temp1, temp2, 0, 0)) != MP_OKAY) {
+        r = MP_INIT_E;
+        goto out;
+    }
 
-    if (mp_iszero(&srp->priv) == MP_YES)
-        return SRP_CALL_ORDER_E;
+    if (!srp || !clientPubKey || clientPubKeySz == 0
+        || !serverPubKey || serverPubKeySz == 0) {
+        r = BAD_FUNC_ARG;
+        goto out;
+    }
+
+#ifdef WOLFSSL_SMALL_STACK
+    if ((hash == NULL) ||
+        (digest == NULL) ||
+        (u == NULL) ||
+        (s == NULL) ||
+        (temp1 == NULL) ||
+        (temp2 == NULL)) {
+        r = MEMORY_E;
+        goto out;
+    }
+#endif
+
+    if (mp_iszero(&srp->priv) == MP_YES) {
+        r = SRP_CALL_ORDER_E;
+        goto out;
+    }
 
     /* initializing variables */
 
-    if ((r = SrpHashInit(&hash, srp->type)) != 0)
-        return r;
+    if ((r = SrpHashInit(hash, srp->type)) != 0)
+        goto out;
 
     digestSz = SrpHashSize(srp->type);
     secretSz = mp_unsigned_bin_size(&srp->N);
 
-    if ((secret = (byte*)XMALLOC(secretSz, srp->heap, DYNAMIC_TYPE_SRP)) ==NULL)
-        return MEMORY_E;
+    if ((secretSz < clientPubKeySz) || (secretSz < serverPubKeySz)) {
+        r = BAD_FUNC_ARG;
+        goto out;
+    }
 
-    if ((r = mp_init_multi(&u, &s, &temp1, &temp2, 0, 0)) != MP_OKAY) {
-        XFREE(secret, srp->heap, DYNAMIC_TYPE_SRP);
-        return r;
+    if ((secret = (byte*)XMALLOC(secretSz, srp->heap, DYNAMIC_TYPE_SRP)) == NULL) {
+        r = MEMORY_E;
+        goto out;
     }
 
     /* building u (random scrambling parameter) */
 
     /* H(A) */
-    for (i = 0; !r && i < secretSz - clientPubKeySz; i++)
-        r = SrpHashUpdate(&hash, &pad, 1);
-    if (!r) r = SrpHashUpdate(&hash, clientPubKey, clientPubKeySz);
+    for (i = 0; i < secretSz - clientPubKeySz; i++) {
+        if ((r = SrpHashUpdate(hash, &pad, 1)))
+            goto out;
+    }
+
+    if ((r = SrpHashUpdate(hash, clientPubKey, clientPubKeySz)))
+        goto out;
 
     /* H(A | B) */
-    for (i = 0; !r && i < secretSz - serverPubKeySz; i++)
-        r = SrpHashUpdate(&hash, &pad, 1);
-    if (!r) r = SrpHashUpdate(&hash, serverPubKey, serverPubKeySz);
+    for (i = 0; i < secretSz - serverPubKeySz; i++) {
+        if ((r = SrpHashUpdate(hash, &pad, 1)))
+            goto out;
+    }
+    if ((r = SrpHashUpdate(hash, serverPubKey, serverPubKeySz)))
+        goto out;
 
     /* set u */
-    if (!r) r = SrpHashFinal(&hash, digest);
-    if (!r) r = mp_read_unsigned_bin(&u, digest, SrpHashSize(srp->type));
+    if ((r = SrpHashFinal(hash, digest)))
+        goto out;
+    if ((r = mp_read_unsigned_bin(u, digest, SrpHashSize(srp->type))))
+        goto out;
+    SrpHashFree(hash);
 
     /* building s (secret) */
 
-    if (!r && srp->side == SRP_CLIENT_SIDE) {
+    if (srp->side == SRP_CLIENT_SIDE) {
 
         /* temp1 = B - k * v; rejects k == 0, B == 0 and B >= N. */
-        r = mp_read_unsigned_bin(&temp1, srp->k, digestSz);
-        if (!r) r = mp_iszero(&temp1) == MP_YES ? SRP_BAD_KEY_E : 0;
-        if (!r) r = mp_exptmod(&srp->g, &srp->auth, &srp->N, &temp2);
-        if (!r) r = mp_mulmod(&temp1, &temp2, &srp->N, &s);
-        if (!r) r = mp_read_unsigned_bin(&temp2, serverPubKey, serverPubKeySz);
-        if (!r) r = mp_iszero(&temp2) == MP_YES ? SRP_BAD_KEY_E : 0;
-        if (!r) r = mp_cmp(&temp2, &srp->N) != MP_LT ? SRP_BAD_KEY_E : 0;
-        if (!r) r = mp_sub(&temp2, &s, &temp1);
+        if ((r = mp_read_unsigned_bin(temp1, srp->k, digestSz)))
+            goto out;
+        if (mp_iszero(temp1) == MP_YES) {
+            r = SRP_BAD_KEY_E;
+            goto out;
+        }
+        if ((r = mp_exptmod(&srp->g, &srp->auth, &srp->N, temp2)))
+            goto out;
+        if ((r = mp_mulmod(temp1, temp2, &srp->N, s)))
+            goto out;
+        if ((r = mp_read_unsigned_bin(temp2, serverPubKey, serverPubKeySz)))
+            goto out;
+        if (mp_iszero(temp2) == MP_YES) {
+            r = SRP_BAD_KEY_E;
+            goto out;
+        }
+        if (mp_cmp(temp2, &srp->N) != MP_LT) {
+            r = SRP_BAD_KEY_E;
+            goto out;
+        }
+        if ((r = mp_submod(temp2, s, &srp->N, temp1)))
+            goto out;
 
         /* temp2 = a + u * x */
-        if (!r) r = mp_mulmod(&u, &srp->auth, &srp->N, &s);
-        if (!r) r = mp_add(&srp->priv, &s, &temp2);
+        if ((r = mp_mulmod(u, &srp->auth, &srp->N, s)))
+            goto out;
+        if ((r = mp_add(&srp->priv, s, temp2)))
+            goto out;
 
         /* secret = temp1 ^ temp2 % N */
-        if (!r) r = mp_exptmod(&temp1, &temp2, &srp->N, &s);
+        if ((r = mp_exptmod(temp1, temp2, &srp->N, s)))
+            goto out;
 
-    } else if (!r && srp->side == SRP_SERVER_SIDE) {
+    } else if (srp->side == SRP_SERVER_SIDE) {
         /* temp1 = v ^ u % N */
-        r = mp_exptmod(&srp->auth, &u, &srp->N, &temp1);
+        if ((r = mp_exptmod(&srp->auth, u, &srp->N, temp1)))
+            goto out;
 
         /* temp2 = A * temp1 % N; rejects A == 0, A >= N */
-        if (!r) r = mp_read_unsigned_bin(&s, clientPubKey, clientPubKeySz);
-        if (!r) r = mp_iszero(&s) == MP_YES ? SRP_BAD_KEY_E : 0;
-        if (!r) r = mp_cmp(&s, &srp->N) != MP_LT ? SRP_BAD_KEY_E : 0;
-        if (!r) r = mp_mulmod(&s, &temp1, &srp->N, &temp2);
+        if ((r = mp_read_unsigned_bin(s, clientPubKey, clientPubKeySz)))
+            goto out;
+        if (mp_iszero(s) == MP_YES) {
+            r = SRP_BAD_KEY_E;
+            goto out;
+        }
+        if (mp_cmp(s, &srp->N) != MP_LT) {
+            r = SRP_BAD_KEY_E;
+            goto out;
+        }
+        if ((r = mp_mulmod(s, temp1, &srp->N, temp2)))
+            goto out;
 
         /* rejects A * v ^ u % N >= 1, A * v ^ u % N == -1 % N */
-        if (!r) r = mp_read_unsigned_bin(&temp1, (const byte*)"\001", 1);
-        if (!r) r = mp_cmp(&temp2, &temp1) != MP_GT ? SRP_BAD_KEY_E : 0;
-        if (!r) r = mp_sub(&srp->N, &temp1, &s);
-        if (!r) r = mp_cmp(&temp2, &s) == MP_EQ ? SRP_BAD_KEY_E : 0;
+        if ((r = mp_read_unsigned_bin(temp1, (const byte*)"\001", 1)))
+            goto out;
+        if (mp_cmp(temp2, temp1) != MP_GT) {
+            r = SRP_BAD_KEY_E;
+            goto out;
+        }
+        if ((r = mp_sub(&srp->N, temp1, s)))
+            goto out;
+        if (mp_cmp(temp2, s) == MP_EQ) {
+            r = SRP_BAD_KEY_E;
+            goto out;
+        }
 
         /* secret = temp2 * b % N */
-        if (!r) r = mp_exptmod(&temp2, &srp->priv, &srp->N, &s);
+        if ((r = mp_exptmod(temp2, &srp->priv, &srp->N, s)))
+            goto out;
     }
 
     /* building session key from secret */
 
-    if (!r) r = mp_to_unsigned_bin(&s, secret);
-    if (!r) r = srp->keyGenFunc_cb(srp, secret, mp_unsigned_bin_size(&s));
+    if ((r = mp_to_unsigned_bin(s, secret)))
+        goto out;
+    if ((r = srp->keyGenFunc_cb(srp, secret, mp_unsigned_bin_size(s))))
+        goto out;
 
     /* updating client proof = H( H(N) ^ H(g) | H(user) | salt | A | B | K) */
 
-    if (!r) r = SrpHashUpdate(&srp->client_proof, clientPubKey, clientPubKeySz);
-    if (!r) r = SrpHashUpdate(&srp->client_proof, serverPubKey, serverPubKeySz);
-    if (!r) r = SrpHashUpdate(&srp->client_proof, srp->key,     srp->keySz);
+    if ((r = SrpHashUpdate(&srp->client_proof, clientPubKey, clientPubKeySz)))
+        goto out;
+    if ((r = SrpHashUpdate(&srp->client_proof, serverPubKey, serverPubKeySz)))
+        goto out;
+    if ((r = SrpHashUpdate(&srp->client_proof, srp->key,     srp->keySz)))
+        goto out;
 
     /* updating server proof = H(A) */
 
-    if (!r) r = SrpHashUpdate(&srp->server_proof, clientPubKey, clientPubKeySz);
+    r = SrpHashUpdate(&srp->server_proof, clientPubKey, clientPubKeySz);
 
-    XFREE(secret, srp->heap, DYNAMIC_TYPE_SRP);
-    mp_clear(&u); mp_clear(&s); mp_clear(&temp1); mp_clear(&temp2);
+  out:
+
+    if (secret) {
+        ForceZero(secret, secretSz);
+        XFREE(secret, srp->heap, DYNAMIC_TYPE_SRP);
+    }
+
+#ifdef WOLFSSL_SMALL_STACK
+    if (hash)
+        XFREE(hash, srp->heap, DYNAMIC_TYPE_SRP);
+    if (digest)
+        XFREE(digest, srp->heap, DYNAMIC_TYPE_SRP);
+    if (u) {
+        if (r != MP_INIT_E)
+            mp_clear(u);
+        XFREE(u, srp->heap, DYNAMIC_TYPE_SRP);
+    }
+    if (s) {
+        if (r != MP_INIT_E)
+            mp_clear(s);
+        XFREE(s, srp->heap, DYNAMIC_TYPE_SRP);
+    }
+    if (temp1) {
+        if (r != MP_INIT_E)
+            mp_clear(temp1);
+        XFREE(temp1, srp->heap, DYNAMIC_TYPE_SRP);
+    }
+    if (temp2) {
+        if (r != MP_INIT_E)
+            mp_clear(temp2);
+        XFREE(temp2, srp->heap, DYNAMIC_TYPE_SRP);
+    }
+#else
+    if (r != MP_INIT_E) {
+        mp_clear(u);
+        mp_clear(s);
+        mp_clear(temp1);
+        mp_clear(temp2);
+    }
+#endif
 
     return r;
 }
