@@ -1,6 +1,6 @@
 /* xil-sha3.h
  *
- * Copyright (C) 2006-2017 wolfSSL Inc.
+ * Copyright (C) 2006-2020 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -34,7 +34,7 @@
 typedef struct Sha3 {
     XSecure_Sha3 hw;
     XCsuDma      dma;
-} Sha3;
+} wc_Sha3;
 
 #ifdef __cplusplus
     } /* extern "C" */
